### ############################# ###
### GMB 2023
### Autor: S. Seifert (University of Göttingen)
###
################################ ###

### Disclaimer
# Diese Datei repliziert die Ergebnisse der Analysen des Beitrags von Hüttel
# und Seifert publiziert in LSA Verm sowie die Ergebnisse im 
# Grundstücksmarktbericht Sachsen-Anhalt 2023

# Zur Replikation wird die Kaufpreissammlung sowie shapefiles zu Windvorranggebieten
# benötigt. Der eingelesene Datensatz enthält alle nötigen Informationen bereits.

### ### ### ### ### ### ### ### ###
### 0 Funktionen und Parameter ####
### ### ### ### ### ### ### ### ###
library(stargazer)
library(lmtest)
library(sandwich)
library(jtools)
library(sf)
library(ggplot2)

# Funktionen für robuste Standardfehler
ctestfn <- function(lmobj){coeftest(lmobj, vcov = vcovHC(lmobj, type = "HC0"))}
se_robust <- function(x){coeftest(x, vcov. = sandwich::sandwich, type = "HC0")[, "Std. Error"]}



### ### ### ## # 
### 1 Daten ####
### ### ### ## # 
### Grunddatensatz
setwd("D:/OC/FORLand/2_Data/Sachsen-Anhalt/02_GA/02_GMB/Analysen GMB 2023")
KDSA <- readRDS("Datensatz_Sachsen_Anhalt_v01.Rds")


### Datensatz für Sonderauswertung Restpacht
KDSA_Sonder <- KDSA[KDSA$Lot_LeasePresence==1,]
KDSA_Sonder <- KDSA_Sonder[is.na(KDSA_Sonder$Lot_LeaseTerm)== FALSE & KDSA_Sonder$Lot_LeaseTerm>=0,]


### Bodenrichtwertzonen: Geometrie
BRWZonen <- read_sf( "D:/OC/FORLand/2_Data/Sachsen-Anhalt/02_GA/02_GMB/Analysen GMB 2023/BRW_Zonen_updated_Okt20/Zonengeometrie.shp")


### ### ### ### ### ### ### ## #
### 2 Deskriptive Statistik ####
### ### ### ### ### ### ### ## #
cols <- c("Price_PerSquareMeter", "Lot_Size", "Lot_QualityArable", "Lot_Independence",
          "Lot_LeasePresence", "nbelastet","pbelastet", "within_WPA",
          "Strikte_Schutzgebiete","HQ100")

### Deskriptive gesamtes Stichprobe
stargazer(KDSA[, cols], type = "text", digits = 2,
  summary.stat = c("mean", "median", "sd", "min","max", "p25", "p75"))

### Deskriptive Sonderauswertung
stargazer(KDSA_Sonder[, c(cols, "Lot_LeaseTerm")], type = "text", digits = 2,
          summary.stat = c("mean", "median", "sd", "min","max", "p25", "p75"))




### ### ### ### ### ### ##
### 3 Modellschätzung ####
### ### ### ### ### ### ##
### vollständige Preisregression
M_GA1<- lm(Price_PerSquareMeterLog ~ I(sqrt(Lot_Size)) 
           + I(sqrt(Lot_QualityArable)) 
           + I(Lot_Size * Lot_QualityArable) 
           + Lot_Independence_num 
           + Lot_LeasePresence
           + Strikte_Schutzgebiete
           + nbelastet
           + pbelastet
           + within_WPA
           + dummy_Q3
           + trend_quarter_2017 + I(trend_quarter_2017^2)  + I(trend_quarter_2017^3) 
           + HQ100
           + factor(BRW_Zone_2020)
           , data = KDSA)

### Sonderauswertung Restpachtdauer
M_GA2<- lm(Price_PerSquareMeterLog ~ I(sqrt(Lot_Size)) 
           + I(sqrt(Lot_QualityArable)) 
           + I(Lot_Size * Lot_QualityArable) 
           + Lot_Independence_num 
           + Lot_LeaseTerm 
           + I(Lot_LeaseTerm^2)
           + dummy_Q3
           + trend_quarter_2017 + I(trend_quarter_2017^2)  + I(trend_quarter_2017^3) 
           + factor(BRW_Zone_2020)
           , data = KDSA_Sonder)


### Regressionsergebnisse ohne/mit robusten Standardfehlern
summary(M_GA1) # R²: 0.7042
ctestfn(M_GA1)

summary(M_GA2) # R²: 0.7042
ctestfn(M_GA2)

stargazer(ctestfn(M_GA1), ctestfn(M_GA2),
          type = "html",
          out="Regression_results_R1_R2.html", 
          report = "vcs",
          ci = TRUE,
          single.row = TRUE)



### Regressionskoeffizientenplot
coefplot_MGA1 <- plot_coefs(M_GA1, coefs = c(
  "√Losgröße" = "I(sqrt(Lot_Size))",
  "√Ackerzahl" = "I(sqrt(Lot_QualityArable))",
  "Losgröße x Ackerzahl" = "I(Lot_Size * Lot_QualityArable * 10^-2)",
  "Losselbstständigkeit" ="Lot_Independence_num",
  "Pachtstatus" = "Lot_LeasePresence" ,
  Schutzgebiet = "Strikte_Schutzgebiete",
  "N-Kulisse" = "nbelastet", 
  "P-Kulisse" = "pbelastet",
  Windvorrang = "within_WPA", 
  HQ100 = "HQ100",
  Trend = "trend_quarter_2017",
  "Trend²" = "I(trend_quarter_2017^2)",
  "Trend³" = "I(trend_quarter_2017^3)"),
  ci_level = 0.95,
  #inner_ci_level = 0.9,
  #plot.distributions = TRUE,
  point.size = 5,
  colors = "black")+ theme(axis.text = element_text(size = 15),
                           axis.text.y = element_text(size = 15)) + xlab("Geschätzter Parameter") 

ggsave("Koeffizientenplot_MGA1.png",coefplot_MGA1)




### ### ### ### ### #
### 4 Auswertung ####
### ### ### ### ### #

### ### ### ### ### ### #
### ** 4a Preistrend ####
### ### ### ### ### ### #
### Erzeugung Datensatz mit Variation nur in Zeitvariablen, 
### Verbleibende Variablen werdenn auf Mean gesetzt.
prediction_data <- M_GA1$model[1,]
prediction_data$`I(sqrt(Lot_Size))` <- sqrt(mean(KDSA$Lot_Size))
prediction_data$`I(sqrt(Lot_QualityArable))` <- sqrt(mean(KDSA$Lot_QualityArable))
prediction_data$`I(Lot_Size * Lot_QualityArable)` <- mean(KDSA$Lot_Size) * mean(KDSA$Lot_QualityArable)
prediction_data$Lot_Size <- mean(KDSA$Lot_Size)
prediction_data$Lot_QualityArable <- mean(KDSA$Lot_QualityArable)

prediction_data <- prediction_data[rep(1:nrow(prediction_data), times = 23), ]
prediction_data$trend_quarter_2017 <- 1:23
prediction_data$trend_quarter_2017_sq <- (1:23)^2
prediction_data$trend_quarter_2017_cub <- (1:23)^3
prediction_data$`I(trend_quarter_2017^2)` <- (1:23)^2
prediction_data$`I(trend_quarter_2017^3)` <- (1:23)^3
prediction_data$BRW_Zone_2020 <- "0267510"

predicted_prices <- exp(predict(M_GA1, prediction_data) + 0.5*sigma(M_GA1))

png("Preistrend_GMB.png", height = 320, width = 400)
plot(prediction_data$trend_quarter_2017,
     predicted_prices,
     xaxt = "n",
     yaxt = "n",
     xlab = "",
     ylab = "Preis in €/m²", 
     bty = "n",
     ylim =c(1.7,2.5),
     type = "b")
axis(1, at = 1:23, labels = c(c("Q1/17", "","Q3/17",""),c("Q1/18", "","Q3/18",""),c("Q1/19", "","Q3/19",""),
                              c("Q1/20", "","Q3/20",""),c("Q1/21", "","Q3/21",""),c("Q1/22", "","Q3/22")),
     las = 2)
axis(2, at = seq(1.7,2.5,0.1), las = 1)
grid()
dev.off()



### ### ### ### ### ### ### ### ## #
### ** 4b Umrechnungskoeffizienten ####
### ### ### ### ### ### ### ### ## #
### Entnommen aus Grundstücksmarktbericht 2023
AZ_vec <- c(20,25,30,35,40,45,50,55,60,65,70,75,80,85,90,95,100)
AZ_UK <- c(68,72,77,81,85,89,92,96,100,104,108,111,115,119,123,126,130)

LotSize_vec <- c(0.25,0.5,0.75,1,5,10,15,20,25,30,35,40,45,50)
LotSize_vec2 <- c(1,2,3,4,5,10,15,20,25,30,35,40,45,50)
LotSize_UK <- c(85,87,88,89,100,108,115,120,125,129,133,136,140,143)

RPD_vec<-c(0,0.5,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18)
RPD_UK<-c(106,105,105,103,101,100,99,98,97,96,95,95,94,94,94,94,94,94,94,95)

p1 <- ggplot(data.frame(x=AZ_vec,y=AZ_UK), aes(x=x,y=y)) + geom_line(size=1.2) 
p2 <- ggplot(data.frame(x=LotSize_vec,y=LotSize_UK), aes(x=x,y=y)) + geom_line()
p3 <- ggplot(data.frame(x=RPD_vec,y=RPD_UK), aes(x=x,y=y)) + geom_line()

tiff("Umrechnungskoeffizienten_AZ.tif", width=480, height=480, res=100)
#layout(matrix(1:3, ncol=3))
par(mar=c(5,4,1,1))
plot(AZ_vec,AZ_UK, type = "b",xaxt="n", 
     xlab = "Ackerzahl", ylab = "Umrechnungskoeffizient", 
     bty="L", ylim =c(65,135), las = 2)
axis(1, at=AZ_vec, las=2)
grid()
dev.off()


tiff("Umrechnungskoeffizienten_size.tif", width=480, height=480, res=100)
par(mar=c(5,4,1,1))
plot(LotSize_vec[c(1,5:14)],LotSize_UK[c(1,5:14)], type = "b",xaxt="n", 
     xlab = "Losgröße (Hektar)", ylab = "Umrechnungskoeffizient", bty="L", ylim =c(85,145))
axis(1, at=LotSize_vec[c(1,5:14)], labels = LotSize_vec[c(1,5:14)], las=2)
grid()
dev.off()


tiff("Umrechnungskoeffizienten_RPD.tif", width=480, height=480, res=100)
par(mar=c(5,4,1,1))
plot(RPD_vec[c(1,3:20)],RPD_UK[c(1,3:20)], type = "b",xaxt="n", 
     xlab = "Restpacht (Jahre)", ylab = "Umrechnungskoeffizient", bty="L", ylim = c(92,108))
axis(1, at=RPD_vec[c(1,3:20)], las=2)
grid()
dev.off()


### ### ### ### ### ### ### ### ## #
### ** 4c Preise für einheitliches Los ####
### ### ### ### ### ### ### ### ## #
BRWZonen$brzo %in% substr(names(coefficients(M_GA1)[16:101]),22,28)
BRWZonen$brzo[1]

# Identische Loscharakteristika für alle Zonen
prediction_data <- M_GA1$model[1,]
prediction_data$Lot_QualityArable <- 70
prediction_data$"I(sqrt(Lot_QualityArable))" <- sqrt(70)
prediction_data$"I(sqrt(Lot_Size))" <- sqrt(5)
prediction_data$Lot_Size <- 5
prediction_data$"I(Lot_Size * Lot_QualityArable)" <- 5*70
prediction_data$BRW_Zone_2020 <- "0327540"
prediction_data$trend_quarter_2017 <- 24
prediction_data$trend_quarter_2017_sq <- 24^2
prediction_data$trend_quarter_2017_cub <- 24^3
prediction_data$Lot_Independence_num <- 1
prediction_data$LotIsLeased_num <- 1


repeated_prediction_data <- prediction_data[rep(1:nrow(prediction_data), times = 87), ]
repeated_prediction_data$BRW_Zone_2020 <- BRWZonen$brzo

# Preisvorhersage
BRWZonen$predicted_prices <- exp(predict(M_GA1, repeated_prediction_data))

# Abbildung
ggplot() + geom_sf(data = BRWZonen, aes(fill = predicted_prices), linewidth = 0.4)+
  scale_fill_viridis_c() +
  theme_void() +
  labs(fill="€/m²") + 
  theme(legend.text=element_text(size=15),legend.title=element_text(size=15))

ggsave("predicted_prices.png")